@@ -8,8 +8,8 @@
 #include <semaphore.h>
 
 #include "androidcontentfileengine.h"
+#include "qandroidapkfileengine.h"
 #include "androiddeadlockprotector.h"
-#include "androidjniaccessibility.h"
 #include "androidjniinput.h"
 #include "androidjnimain.h"
 #include "androidjnimenu.h"
@@ -18,7 +18,12 @@
 #include "qandroideventdispatcher.h"
 #include "qandroidplatformdialoghelpers.h"
 #include "qandroidplatformintegration.h"
+#if QT_CONFIG(clipboard)
 #include "qandroidplatformclipboard.h"
+#endif
+#if QT_CONFIG(accessibility)
+#include "androidjniaccessibility.h"
+#endif
 #include "qandroidplatformwindow.h"
 
 #include <android/api-level.h>
@@ -45,10 +50,7 @@ using namespace Qt::StringLiterals;
 
 QT_BEGIN_NAMESPACE
 
-static JavaVM *m_javaVM = nullptr;
 static jclass m_applicationClass  = nullptr;
-static jobject m_classLoaderObject = nullptr;
-static jmethodID m_loadClassMethodID = nullptr;
 static AAssetManager *m_assetManager = nullptr;
 static jobject m_assets = nullptr;
 static jobject m_resourcesObj = nullptr;
@@ -83,6 +85,7 @@ static double m_density = 1.0;
 
 static AndroidAssetsFileEngineHandler *m_androidAssetsFileEngineHandler = nullptr;
 static AndroidContentFileEngineHandler *m_androidContentFileEngineHandler = nullptr;
+static QAndroidApkFileEngineHandler *m_androidApkFileEngineHandler = nullptr;
 
 static AndroidBackendRegister *m_backendRegister = nullptr;
 
@@ -92,8 +95,9 @@ static const char m_methodErrorMsg[] = "Can't find method \"%s%s\"";
 
 Q_CONSTINIT static QBasicAtomicInt startQtAndroidPluginCalled = Q_BASIC_ATOMIC_INITIALIZER(0);
 
-Q_DECLARE_JNI_CLASS(QtWindowInterface, "org/qtproject/qt/android/QtWindowInterface")
+#if QT_CONFIG(accessibility)
 Q_DECLARE_JNI_CLASS(QtAccessibilityInterface, "org/qtproject/qt/android/QtAccessibilityInterface");
+#endif
 
 namespace QtAndroid
 {
@@ -166,11 +170,6 @@ namespace QtAndroid
         return m_density;
     }
 
-    JavaVM *javaVM()
-    {
-        return m_javaVM;
-    }
-
     AAssetManager *assetManager()
     {
         return m_assetManager;
@@ -181,14 +180,6 @@ namespace QtAndroid
         return m_applicationClass;
     }
 
-    // TODO move calls from here to where they logically belong
-    void setSystemUiVisibility(SystemUiVisibility uiVisibility)
-    {
-        AndroidBackendRegister *reg = QtAndroid::backendRegister();
-        reg->callInterface<QtJniTypes::QtWindowInterface, void>("setSystemUiVisibility",
-                                                                jint(uiVisibility));
-    }
-
     bool isQtApplication()
     {
         // Returns true if the app is a Qt app, i.e. Qt controls the whole app and
@@ -207,6 +198,7 @@ namespace QtAndroid
         return true;
     }
 
+#if QT_CONFIG(accessibility)
     void initializeAccessibility()
     {
         m_backendRegister->callInterface<QtJniTypes::QtAccessibilityInterface, void>(
@@ -248,6 +240,7 @@ namespace QtAndroid
         m_backendRegister->callInterface<QtJniTypes::QtAccessibilityInterface, void>(
                 "notifyScrolledEvent", accessibilityObjectId);
     }
+#endif //QT_CONFIG(accessibility)
 
     void notifyNativePluginIntegrationReady(bool ready)
     {
@@ -377,13 +370,29 @@ namespace QtAndroid
 
 } // namespace QtAndroid
 
+static bool initJavaReferences(QJniEnvironment &env);
+
 static jboolean startQtAndroidPlugin(JNIEnv *env, jobject /*object*/, jstring paramsString)
 {
     Q_UNUSED(env)
+    // Init all the Java refs, if they haven't already been initialized. They get initialized
+    // when the library is loaded, but in case Qt is terminated, they are cleared, and in case
+    // Qt is then started again JNI_OnLoad will not be called again, since the library is already
+    // loaded - in that case we need to init again here, hence the check.
+    // TODO QTBUG-130614 QtCore also inits some Java references in qjnihelpers - we probably
+    // want to reset those, too.
+    QJniEnvironment qEnv;
+    if (!qEnv.isValid()) {
+        __android_log_print(ANDROID_LOG_FATAL, "Qt", "Failed to initialize the JNI Environment");
+        return JNI_ERR;
+    }
+    if (!initJavaReferences(qEnv))
+        return false;
 
     m_androidPlatformIntegration = nullptr;
     m_androidAssetsFileEngineHandler = new AndroidAssetsFileEngineHandler();
     m_androidContentFileEngineHandler = new AndroidContentFileEngineHandler();
+    m_androidApkFileEngineHandler = new QAndroidApkFileEngineHandler();
     m_mainLibraryHnd = nullptr;
     m_backendRegister = new AndroidBackendRegister();
 
@@ -470,7 +479,7 @@ static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
             qWarning() << "dlclose failed:" << dlerror();
     }
 
-    if (m_applicationClass)
+    if (m_applicationClass && QtAndroid::isQtApplication())
         QJniObject::callStaticMethod<void>(m_applicationClass, "quitApp", "()V");
 
     sem_post(&m_terminateSemaphore);
@@ -496,6 +505,48 @@ static void quitQtAndroidPlugin(JNIEnv *env, jclass /*clazz*/)
     m_androidAssetsFileEngineHandler = nullptr;
     delete m_androidContentFileEngineHandler;
     m_androidContentFileEngineHandler = nullptr;
+    delete m_androidApkFileEngineHandler;
+    m_androidApkFileEngineHandler = nullptr;
+}
+
+static void clearJavaReferences(JNIEnv *env)
+{
+    if (m_applicationClass) {
+        env->DeleteGlobalRef(m_applicationClass);
+        m_applicationClass = nullptr;
+    }
+    if (m_resourcesObj) {
+        env->DeleteGlobalRef(m_resourcesObj);
+        m_resourcesObj = nullptr;
+    }
+    if (m_bitmapClass) {
+        env->DeleteGlobalRef(m_bitmapClass);
+        m_bitmapClass = nullptr;
+    }
+    if (m_ARGB_8888_BitmapConfigValue) {
+        env->DeleteGlobalRef(m_ARGB_8888_BitmapConfigValue);
+        m_ARGB_8888_BitmapConfigValue = nullptr;
+    }
+    if (m_RGB_565_BitmapConfigValue) {
+        env->DeleteGlobalRef(m_RGB_565_BitmapConfigValue);
+        m_RGB_565_BitmapConfigValue = nullptr;
+    }
+    if (m_bitmapDrawableClass) {
+        env->DeleteGlobalRef(m_bitmapDrawableClass);
+        m_bitmapDrawableClass = nullptr;
+    }
+    if (m_assets) {
+        env->DeleteGlobalRef(m_assets);
+        m_assets = nullptr;
+    }
+    if (m_qtActivityClass) {
+        env->DeleteGlobalRef(m_qtActivityClass);
+        m_qtActivityClass = nullptr;
+    }
+    if (m_qtServiceClass) {
+        env->DeleteGlobalRef(m_qtServiceClass);
+        m_qtServiceClass = nullptr;
+    }
 }
 
 static void terminateQt(JNIEnv *env, jclass /*clazz*/)
@@ -512,29 +563,15 @@ static void terminateQt(JNIEnv *env, jclass /*clazz*/)
 
     sem_destroy(&m_terminateSemaphore);
 
-    env->DeleteGlobalRef(m_applicationClass);
-    env->DeleteGlobalRef(m_classLoaderObject);
-    if (m_resourcesObj)
-        env->DeleteGlobalRef(m_resourcesObj);
-    if (m_bitmapClass)
-        env->DeleteGlobalRef(m_bitmapClass);
-    if (m_ARGB_8888_BitmapConfigValue)
-        env->DeleteGlobalRef(m_ARGB_8888_BitmapConfigValue);
-    if (m_RGB_565_BitmapConfigValue)
-        env->DeleteGlobalRef(m_RGB_565_BitmapConfigValue);
-    if (m_bitmapDrawableClass)
-        env->DeleteGlobalRef(m_bitmapDrawableClass);
-    if (m_assets)
-        env->DeleteGlobalRef(m_assets);
-    if (m_qtActivityClass)
-        env->DeleteGlobalRef(m_qtActivityClass);
-    if (m_qtServiceClass)
-        env->DeleteGlobalRef(m_qtServiceClass);
+    clearJavaReferences(env);
+
     m_androidPlatformIntegration = nullptr;
     delete m_androidAssetsFileEngineHandler;
     m_androidAssetsFileEngineHandler = nullptr;
     delete m_androidContentFileEngineHandler;
     m_androidContentFileEngineHandler = nullptr;
+    delete m_androidApkFileEngineHandler;
+    m_androidApkFileEngineHandler = nullptr;
     delete m_backendRegister;
     m_backendRegister = nullptr;
     sem_post(&m_exitSemaphore);
@@ -574,26 +611,6 @@ static void setDisplayMetrics(JNIEnv * /*env*/, jclass /*clazz*/, jint screenWid
 }
 Q_DECLARE_JNI_NATIVE_METHOD(setDisplayMetrics)
 
-static void updateWindow(JNIEnv */*env*/, jobject /*thiz*/)
-{
-    if (!m_androidPlatformIntegration)
-        return;
-
-    if (QGuiApplication::instance() != nullptr) {
-        const auto tlw = QGuiApplication::topLevelWindows();
-        for (QWindow *w : tlw) {
-
-            // Skip non-platform windows, e.g., offscreen windows.
-            if (!w->handle())
-                continue;
-
-            QRect availableGeometry = w->screen()->availableGeometry();
-            if (w->geometry().width() > 0 && w->geometry().height() > 0 && availableGeometry.width() > 0 && availableGeometry.height() > 0)
-                QWindowSystemInterface::handleExposeEvent(w, QRegion(QRect(QPoint(), w->geometry().size())));
-        }
-    }
-}
-
 static void updateApplicationState(JNIEnv */*env*/, jobject /*thiz*/, jint state)
 {
     QMutexLocker lock(&m_platformMutex);
@@ -738,7 +755,6 @@ static JNINativeMethod methods[] = {
     { "quitQtCoreApplication", "()V", (void *)quitQtCoreApplication },
     { "terminateQt", "()V", (void *)terminateQt },
     { "waitForServiceSetup", "()V", (void *)waitForServiceSetup },
-    { "updateWindow", "()V", (void *)updateWindow },
     { "updateApplicationState", "(I)V", (void *)updateApplicationState },
     { "onActivityResult", "(IILandroid/content/Intent;)V", (void *)onActivityResult },
     { "onNewIntent", "(Landroid/content/Intent;)V", (void *)onNewIntent },
@@ -785,17 +801,9 @@ Q_DECLARE_JNI_CLASS(QtDisplayManager, "org/qtproject/qt/android/QtDisplayManager
 
 static bool registerNatives(QJniEnvironment &env)
 {
-    jclass clazz;
-    FIND_AND_CHECK_CLASS("org/qtproject/qt/android/QtNative");
-    m_applicationClass = static_cast<jclass>(env->NewGlobalRef(clazz));
-
-    if (!env.registerNativeMethods(m_applicationClass,
-                                   methods, sizeof(methods) / sizeof(methods[0]))) {
-        __android_log_print(ANDROID_LOG_FATAL,"Qt", "RegisterNatives failed");
-        return false;
-    }
-
-    bool success = env.registerNativeMethods(
+    bool success = env.registerNativeMethods(m_applicationClass,
+                   methods, sizeof(methods) / sizeof(methods[0]));
+    success &= env.registerNativeMethods(
             QtJniTypes::Traits<QtJniTypes::QtDisplayManager>::className(),
             {
                     Q_JNI_NATIVE_METHOD(setDisplayMetrics),
@@ -807,13 +815,38 @@ static bool registerNatives(QJniEnvironment &env)
                     Q_JNI_NATIVE_METHOD(handleUiDarkModeChanged)
             });
 
-    if (!success) {
-        qCritical() << "QtDisplayManager: registerNativeMethods() failed";
-        return JNI_FALSE;
-    }
+    success = success
+        && QtAndroidInput::registerNatives(env)
+        && QtAndroidMenu::registerNatives(env)
+#if QT_CONFIG(accessibility)
+        && QtAndroidAccessibility::registerNatives(env)
+#endif
+        && QtAndroidDialogHelpers::registerNatives(env)
+#if QT_CONFIG(clipboard)
+        && QAndroidPlatformClipboard::registerNatives(env)
+#endif
+        && QAndroidPlatformWindow::registerNatives(env)
+        && QtAndroidWindowEmbedding::registerNatives(env)
+        && AndroidBackendRegister::registerNatives()
+        && QAndroidModelIndexProxy::registerNatives(env)
+        && QAndroidItemModelProxy::registerAbstractNatives(env)
+        && QAndroidItemModelProxy::registerProxyNatives(env);
+
+    return success;
+}
+
+static bool initJavaReferences(QJniEnvironment &env)
+{
+    if (m_applicationClass)
+        return true;
+
+    jclass clazz;
+    FIND_AND_CHECK_CLASS("org/qtproject/qt/android/QtNative");
+    m_applicationClass = static_cast<jclass>(env->NewGlobalRef(clazz));
 
     jmethodID methodID;
     GET_AND_CHECK_STATIC_METHOD(methodID, m_applicationClass, "activity", "()Landroid/app/Activity;");
+
     jobject contextObject = env->CallStaticObjectMethod(m_applicationClass, methodID);
     if (!contextObject) {
         GET_AND_CHECK_STATIC_METHOD(methodID, m_applicationClass, "service", "()Landroid/app/Service;");
@@ -828,11 +861,6 @@ static bool registerNatives(QJniEnvironment &env)
         env->DeleteLocalRef(contextObject);
     });
 
-    GET_AND_CHECK_STATIC_METHOD(methodID, m_applicationClass, "classLoader", "()Ljava/lang/ClassLoader;");
-    m_classLoaderObject = env->NewGlobalRef(env->CallStaticObjectMethod(m_applicationClass, methodID));
-    clazz = env->GetObjectClass(m_classLoaderObject);
-    GET_AND_CHECK_METHOD(m_loadClassMethodID, clazz, "loadClass", "(Ljava/lang/String;)Ljava/lang/Class;");
-
     FIND_AND_CHECK_CLASS("android/content/ContextWrapper");
     GET_AND_CHECK_METHOD(methodID, clazz, "getAssets", "()Landroid/content/res/AssetManager;");
     m_assets = env->NewGlobalRef(env->CallObjectMethod(contextObject, methodID));
@@ -863,12 +891,17 @@ static bool registerNatives(QJniEnvironment &env)
     FIND_AND_CHECK_CLASS("org/qtproject/qt/android/QtServiceBase");
     m_qtServiceClass = static_cast<jclass>(env->NewGlobalRef(clazz));
 
+    // The current thread will be the Qt thread, name it accordingly
+    QThread::currentThread()->setObjectName("QtMainLoopThread");
+
+    QWindowSystemInterfacePrivate::TabletEvent::setPlatformSynthesizesMouse(false);
+
     return true;
 }
 
 QT_END_NAMESPACE
 
-Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void */*reserved*/)
+Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM */*vm*/, void */*reserved*/)
 {
     static bool initialized = false;
     if (initialized)
@@ -876,33 +909,21 @@ Q_DECL_EXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void */*reserved*/)
     initialized = true;
 
     QT_USE_NAMESPACE
-    m_javaVM = vm;
+
     QJniEnvironment env;
     if (!env.isValid()) {
-        m_javaVM = nullptr;
         __android_log_print(ANDROID_LOG_FATAL, "Qt", "Failed to initialize the JNI Environment");
-        return -1;
+        return JNI_ERR;
     }
 
-    if (!registerNatives(env)
-            || !QtAndroidInput::registerNatives(env)
-            || !QtAndroidMenu::registerNatives(env)
-            || !QtAndroidAccessibility::registerNatives(env)
-            || !QtAndroidDialogHelpers::registerNatives(env)
-            || !QAndroidPlatformClipboard::registerNatives(env)
-            || !QAndroidPlatformWindow::registerNatives(env)
-            || !QtAndroidWindowEmbedding::registerNatives(env)
-            || !AndroidBackendRegister::registerNatives()
-            || !QAndroidModelIndexProxy::registerNatives(env)
-            || !QAndroidItemModelProxy::registerAbstractNatives(env)
-            || !QAndroidItemModelProxy::registerProxyNatives(env)) {
+    if (!initJavaReferences(env))
+        return JNI_ERR;
+
+    if (!registerNatives(env)) {
         __android_log_print(ANDROID_LOG_FATAL, "Qt", "registerNatives failed");
-        return -1;
+        return JNI_ERR;
     }
-    QWindowSystemInterfacePrivate::TabletEvent::setPlatformSynthesizesMouse(false);
 
-    // attach qt main thread data to this thread
-    QThread::currentThread()->setObjectName("QtMainLoopThread");
     __android_log_print(ANDROID_LOG_INFO, "Qt", "qt started");
     return JNI_VERSION_1_6;
 }
