@@ -80,7 +80,7 @@ void tst_QRhiWidget::testData()
     }
 #endif
 
-#if defined(Q_OS_MACOS) || defined(Q_OS_IOS)
+#if QT_CONFIG(metal)
     QRhiMetalInitParams metalInitParams;
     if (QRhi::probe(QRhi::Metal, &metalInitParams))
         QTest::newRow("Metal") << QRhiWidget::Api::Metal;
@@ -398,7 +398,7 @@ void tst_QRhiWidget::simple()
                             readResult.pixelSize.width(), readResult.pixelSize.height(),
                             QImage::Format_RGBA8888);
         if (rhi->isYUpInFramebuffer())
-            resultOne = wrapperImage.mirrored();
+            resultOne = wrapperImage.flipped();
         else
             resultOne = wrapperImage.copy();
 
@@ -481,7 +481,7 @@ void tst_QRhiWidget::msaa()
                             QImage::Format_RGBA8888);
         QImage result;
         if (rhi->isYUpInFramebuffer())
-            result = wrapperImage.mirrored();
+            result = wrapperImage.flipped();
         else
             result = wrapperImage.copy();
 
@@ -726,7 +726,7 @@ void tst_QRhiWidget::grabFramebufferWhileStillInvisible()
                             readResult.pixelSize.width(), readResult.pixelSize.height(),
                             QImage::Format_RGBA8888);
         if (w.rhi()->isYUpInFramebuffer())
-            image = wrapperImage.mirrored();
+            image = wrapperImage.flipped();
         else
             image = wrapperImage.copy();
         QRgb c = image.pixel(image.width() / 2, image.height() / 2);
@@ -810,7 +810,7 @@ void tst_QRhiWidget::mirror()
                             QImage::Format_RGBA8888);
         QImage image;
         if (rhi->isYUpInFramebuffer())
-            image = wrapperImage.mirrored();
+            image = wrapperImage.flipped();
         else
             image = wrapperImage.copy();
 
